@@ -28,7 +28,6 @@
 #include "downloader-view.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-adblock-manager.h"
-#include "ephy-command-manager.h"
 #include "ephy-debug.h"
 #include "ephy-embed.h"
 #include "ephy-embed-event.h"
@@ -65,56 +64,7 @@ struct EphyEmbedPrivate
   guint is_setting_zoom : 1;
 };
 
-static void
-impl_manager_do_command (EphyCommandManager *manager,
-                         const char *command)
-{
-  WebKitWebView *web_view = EPHY_EMBED (manager)->priv->web_view;
-
-  if (! strcmp (command, "cmd_copy"))
-    return webkit_web_view_copy_clipboard (web_view);
-  else if (! strcmp (command, "cmd_cut"))
-    return webkit_web_view_cut_clipboard (web_view);
-  else if (! strcmp (command, "cmd_paste"))
-    return webkit_web_view_paste_clipboard (web_view);
-  else if (! strcmp (command, "cmd_selectAll"))
-    return webkit_web_view_select_all (web_view);
-  else if (! strcmp (command, "cmd_undo"))
-    return webkit_web_view_undo (web_view);
-  else if (! strcmp (command, "cmd_redo"))
-    return webkit_web_view_redo (web_view);
-}
-
-static gboolean
-impl_manager_can_do_command (EphyCommandManager *manager,
-                             const char *command)
-{
-  WebKitWebView *web_view = EPHY_EMBED (manager)->priv->web_view;
-
-  if (! strcmp (command, "cmd_copy"))
-    return webkit_web_view_can_copy_clipboard (web_view);
-  else if (! strcmp (command, "cmd_cut"))
-    return webkit_web_view_can_cut_clipboard (web_view);
-  else if (! strcmp (command, "cmd_paste"))
-    return webkit_web_view_can_paste_clipboard (web_view);
-  else if (! strcmp (command, "cmd_undo"))
-    return webkit_web_view_can_undo (web_view);
-  else if (! strcmp (command, "cmd_redo"))
-    return webkit_web_view_can_redo (web_view);
-
-  return FALSE;
-}
-
-static void
-ephy_command_manager_iface_init (EphyCommandManagerIface *iface)
-{
-  iface->do_command = impl_manager_do_command;
-  iface->can_do_command = impl_manager_can_do_command;
-}
-
-G_DEFINE_TYPE_WITH_CODE (EphyEmbed, ephy_embed, GTK_TYPE_SCROLLED_WINDOW,
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_COMMAND_MANAGER,
-                                                ephy_command_manager_iface_init))
+G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
 uri_changed_cb (WebKitWebView *web_view,
@@ -204,18 +154,18 @@ resource_request_starting_cb (WebKitWebView *web_view,
                               WebKitNetworkResponse *response,
                               EphyEmbed *embed)
 {
-  EphyAdBlockManager *adblock_manager = EPHY_ADBLOCK_MANAGER(ephy_embed_shell_get_adblock_manager (embed_shell));
-  const char* uri = webkit_network_request_get_uri(request);
+  EphyAdBlockManager *adblock_manager = EPHY_ADBLOCK_MANAGER (ephy_embed_shell_get_adblock_manager (embed_shell));
+  const char *uri = webkit_network_request_get_uri (request);
 
   /* FIXME: How do we implement the other CHECK_TYPEs?  Perhaps we
    * should figure out a way of adding more information about what the
    * resource is for to WebResource? */
-  if(!ephy_adblock_manager_should_load(adblock_manager, embed, uri,
-                                       AD_URI_CHECK_TYPE_OTHER)) {
+  if (!ephy_adblock_manager_should_load (adblock_manager, embed, uri,
+                                         AD_URI_CHECK_TYPE_OTHER)) {
     g_signal_emit_by_name (EPHY_WEB_VIEW (web_view),
                            "content-blocked", uri);
 
-    webkit_network_request_set_uri(request, "about:blank");
+    webkit_network_request_set_uri (request, "about:blank");
   }
 }
 
@@ -487,7 +437,7 @@ request_destination_uri (WebKitWebView *web_view,
    * WebView or other widget as a parent window.
    */
   window = gtk_widget_get_toplevel (GTK_WIDGET (web_view));
-  if (!GTK_WIDGET_TOPLEVEL (window))
+  if (!gtk_widget_is_toplevel (window))
     window = NULL;
 
   dialog = ephy_file_chooser_new (_("Save"),
@@ -504,12 +454,60 @@ request_destination_uri (WebKitWebView *web_view,
   gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
+/* From the old embed/mozilla/MozDownload.cpp */
+static const char*
+file_is_compressed (const char *filename)
+{
+  int i;
+  static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+
+  for (i = 0; compression[i] != NULL; i++) {
+    if (g_str_has_suffix (filename, compression[i]))
+      return compression[i];
+  }
+
+  return NULL;
+}
+
+static const char*
+parse_extension (const char *filename)
+{
+  const char *compression;
+
+  compression = file_is_compressed (filename);
+
+  /* if the file is compressed we might have a double extension */
+  if (compression != NULL) {
+    int i;
+    static const char * const extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
+
+    for (i = 0; extensions[i] != NULL; i++) {
+      char *suffix;
+      suffix = g_strdup_printf (".%s%s", extensions[i], compression);
+
+      if (g_str_has_suffix (filename, suffix)) {
+        char *p;
+
+        p = g_strrstr (filename, suffix);
+        g_free (suffix);
+
+        return p;
+      }
+
+      g_free (suffix);
+    }
+  }
+
+  /* no compression, just look for the last dot in the filename */
+  return g_strrstr (filename, ".");
+}
+
 static gboolean
 define_destination_uri (WebKitDownload *download,
                         gboolean temporary)
 {
   char *tmp_dir;
-  char *tmp_filename;
+  char *destination_filename;
   char *destination_uri;
   const char *suggested_filename;
 
@@ -532,13 +530,42 @@ define_destination_uri (WebKitDownload *download,
     return FALSE;
   }
 
-  tmp_filename = g_build_filename (tmp_dir, suggested_filename, NULL);
-  destination_uri = g_strconcat ("file://", tmp_filename, NULL);
+  destination_filename = g_build_filename (tmp_dir, suggested_filename, NULL);
+
+  if (g_file_test (destination_filename, G_FILE_TEST_EXISTS)) {
+    int i = 1;
+    const char *dot_pos;
+    gssize position;
+    char *serial = NULL;
+    GString *tmp_filename;
+
+    dot_pos = parse_extension (destination_filename);
+    if (dot_pos)
+      position = dot_pos - destination_filename;
+    else
+      position = strlen (destination_filename);
+
+    tmp_filename = g_string_new (NULL);
+
+    do {
+      serial = g_strdup_printf ("(%d)", i++);
+
+      g_string_assign (tmp_filename, destination_filename);
+      g_string_insert (tmp_filename, position, serial);
+
+      g_free (serial);
+    } while (g_file_test (tmp_filename->str, G_FILE_TEST_EXISTS));
+
+    destination_filename = g_strdup (tmp_filename->str);
+    g_string_free (tmp_filename, TRUE);
+  }
+
+  destination_uri = g_strconcat ("file://", destination_filename, NULL);
 
   webkit_download_set_destination_uri (download, destination_uri);
 
   g_free (tmp_dir);
-  g_free (tmp_filename);
+  g_free (destination_filename);
   g_free (destination_uri);
 
   return TRUE;
@@ -618,27 +645,38 @@ confirm_action_from_mime (WebKitWebView *web_view,
   GAppInfo *helper_app;
   const char *suggested_filename;
   int default_response;
+  WebKitNetworkResponse *response;
+  SoupMessage *message;
 
   parent_window = gtk_widget_get_toplevel (GTK_WIDGET(web_view));
-  if (!GTK_WIDGET_TOPLEVEL (parent_window))
+  if (!gtk_widget_is_toplevel (parent_window))
     parent_window = NULL;
 
-  /* FIXME: we still have no way of getting the content type from
-   * webkit yet; we need to have a proper WebKitNetworkRequest
-   * implementation to do this here; see
-   * https://bugs.webkit.org/show_bug.cgi?id=18608
-   */
   helper_app = NULL;
   mime_description = NULL;
   mime_permission = EPHY_MIME_PERMISSION_SAFE;
+
+  response = webkit_download_get_network_response (download);
+  message = webkit_network_response_get_message (response);
+
+  if (message) {
+    const char *content_type = soup_message_headers_get_content_type (message->response_headers, NULL);
+
+    if (content_type) {
+      mime_description = g_content_type_get_description (content_type);
+      helper_app = g_app_info_get_default_for_type (content_type, FALSE);
+
+      if (helper_app) {
+        action = DOWNLOAD_ACTION_OPEN;
+      }
+    }
+  }
+
   if (mime_description == NULL) {
     mime_description = g_strdup (C_("file type", "Unknown"));
     action = DOWNLOAD_ACTION_OPEN_LOCATION;
   }
 
-  /* OPEN will never happen here, for now; see comment about
-   * WebKitNetworkRequest above!
-   */
   action_label = (action == DOWNLOAD_ACTION_OPEN) ? GTK_STOCK_OPEN : STOCK_DOWNLOAD;
   suggested_filename = webkit_download_get_suggested_filename (download);
 
@@ -711,6 +749,29 @@ confirm_action_from_mime (WebKitWebView *web_view,
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
+static void
+download_status_changed_cb (GObject *object,
+                            GParamSpec *pspec,
+                            WebKitWebView *web_view)
+{
+  WebKitDownload *download = WEBKIT_DOWNLOAD (object);
+
+  g_return_if_fail (webkit_download_get_status (download) == WEBKIT_DOWNLOAD_STATUS_STARTED);
+
+  g_signal_handlers_disconnect_by_func (download,
+                                        download_status_changed_cb,
+                                        web_view);
+
+  /* Is auto-download enabled? */
+  if (eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS)) {
+    perform_auto_download (download);
+    return;
+  }
+
+  g_object_ref (download); /* balanced in confirm_action_response_cb */
+  confirm_action_from_mime (web_view, download, DOWNLOAD_ACTION_DOWNLOAD);
+}
+
 static gboolean
 download_requested_cb (WebKitWebView *web_view,
                        WebKitDownload *download)
@@ -719,11 +780,11 @@ download_requested_cb (WebKitWebView *web_view,
   if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK))
     return FALSE;
 
-  /* Is auto-download enabled? */
-  if (eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS)) {
-    perform_auto_download (download);
-    return TRUE;
-  }
+  /* Wait for the request to be sent in all cases, so that we have a
+   * response which may contain a suggested filename */
+  g_signal_connect (download, "notify::status",
+                    G_CALLBACK (download_status_changed_cb),
+                    web_view);
 
   /* If we are not performing an auto-download, we will ask the user
    * where they want the file to go to; we will start downloading to a
@@ -732,14 +793,6 @@ download_requested_cb (WebKitWebView *web_view,
   if (!define_destination_uri (download, TRUE))
     return FALSE;
 
-  /* FIXME: when we are able to obtain the MIME information from
-   * WebKit, we will want to decide earlier whether we want to open or
-   * open the location to where the file was downloaded. See
-   * perform_auto_download, too.
-   */
-  g_object_ref (download); /* balanced in confirm_action_response_cb */
-  confirm_action_from_mime (web_view, download, DOWNLOAD_ACTION_DOWNLOAD);
-
   return TRUE;
 }
 
