@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
- *  Copyright © 2008 Gustavo Noronha Silva
+ *  Copyright © 2008, 2009 Gustavo Noronha Silva
  *  Copyright © 2009 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -54,7 +54,6 @@ static void     ephy_web_view_init         (EphyWebView *gs);
 #define EMPTY_PAGE              _("Blank page") /* Title for the empty page */
 
 struct _EphyWebViewPrivate {
-  EphyWebViewAddressExpire address_expire;
   EphyWebViewSecurityLevel security_level;
   EphyWebViewDocumentType document_type;
   EphyWebViewNavigationFlags nav_flags;
@@ -67,13 +66,12 @@ struct _EphyWebViewPrivate {
   char *address;
   char *typed_address;
   char *title;
-  int cur_requests;
-  int total_requests;
   char *loading_title;
   char *status_message;
   char *link_message;
   char *icon_address;
   GdkPixbuf *icon;
+  gboolean expire_address_now;
 
   /* File watch */
   GFileMonitor *monitor;
@@ -81,6 +79,9 @@ struct _EphyWebViewPrivate {
   guint reload_scheduled_id;
   guint reload_delay_ticks;
 
+  /* Regex to figure out if we're dealing with a wanna-be URI */
+  GRegex *non_search_regex;
+
   GSList *hidden_popups;
   GSList *shown_popups;
 };
@@ -162,6 +163,7 @@ popups_manager_new_window_info (EphyEmbedContainer *container)
 {
   EphyEmbed *embed;
   EphyWebViewChrome chrome;
+  GtkAllocation allocation;
   gboolean is_popup;
   char *features;
 
@@ -171,10 +173,12 @@ popups_manager_new_window_info (EphyEmbedContainer *container)
   embed = ephy_embed_container_get_active_child (container);
   g_return_val_if_fail (embed != NULL, g_strdup (""));
 
+  gtk_widget_get_allocation (GTK_WIDGET (embed), &allocation);
+
   features = g_strdup_printf
              ("width=%d,height=%d,menubar=%d,status=%d,toolbar=%d",
-              GTK_WIDGET (embed)->allocation.width,
-              GTK_WIDGET (embed)->allocation.height,
+              allocation.width,
+              allocation.height,
               (chrome & EPHY_WEB_VIEW_CHROME_MENUBAR) > 0,
               (chrome & EPHY_WEB_VIEW_CHROME_STATUSBAR) > 0,
               (chrome & EPHY_WEB_VIEW_CHROME_TOOLBAR) > 0);
@@ -440,8 +444,7 @@ ephy_web_view_set_property (GObject *object,
       ephy_web_view_set_popups_allowed (EPHY_WEB_VIEW (object), g_value_get_boolean (value));
       break;
     case PROP_TYPED_ADDRESS:
-      ephy_web_view_set_typed_address (EPHY_WEB_VIEW (object), g_value_get_string (value),
-                                       EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW);
+      ephy_web_view_set_typed_address (EPHY_WEB_VIEW (object), g_value_get_string (value));
       break;
       break;
     case PROP_ADDRESS:
@@ -483,6 +486,22 @@ ephy_web_view_file_monitor_cancel (EphyWebView *view)
   priv->reload_delay_ticks = 0;
 }
 
+static gboolean
+ephy_web_view_key_press_event (GtkWidget *widget, GdkEventKey *event)
+{
+    EphyWebView *web_view = EPHY_WEB_VIEW (widget);
+    gboolean key_handled = FALSE;
+
+    key_handled = GTK_WIDGET_CLASS (ephy_web_view_parent_class)->key_press_event (widget, event);
+
+    if (key_handled)
+      return TRUE;
+
+    g_signal_emit_by_name (web_view, "search-key-press", event, &key_handled);
+
+    return key_handled;
+}
+
 static void
 ephy_web_view_dispose (GObject *object)
 {
@@ -501,6 +520,11 @@ ephy_web_view_finalize (GObject *object)
     priv->icon = NULL;
   }
 
+  if (priv->non_search_regex != NULL) {
+    g_regex_unref (priv->non_search_regex);
+    priv->non_search_regex = NULL;
+  }
+
   ephy_web_view_popups_manager_reset (EPHY_WEB_VIEW (object));
 
   g_free (priv->address);
@@ -518,12 +542,15 @@ static void
 ephy_web_view_class_init (EphyWebViewClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   gobject_class->dispose = ephy_web_view_dispose;
   gobject_class->finalize = ephy_web_view_finalize;
   gobject_class->get_property = ephy_web_view_get_property;
   gobject_class->set_property = ephy_web_view_set_property;
 
+  widget_class->key_press_event = ephy_web_view_key_press_event;
+
   g_object_class_install_property (gobject_class,
                                    PROP_ADDRESS,
                                    g_param_spec_string ("address",
@@ -664,24 +691,7 @@ ephy_web_view_class_init (EphyWebViewClass *klass)
             G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
             G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
             G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
-/**
- * EphyWebView::ge-context-menu:
- * @embed:
- * @event: the #EphyEmbedEvent which triggered this signal
- *
- * The ::ge_context_menu signal is emitted when a context menu is to be
- * displayed. This will usually happen when the user right-clicks on a part of
- * @embed.
- **/
-    g_signal_new ("ge_context_menu",
-            EPHY_TYPE_WEB_VIEW,
-            G_SIGNAL_RUN_LAST,
-            G_STRUCT_OFFSET (EphyWebViewClass, context_menu),
-            g_signal_accumulator_true_handled, NULL,
-            ephy_marshal_BOOLEAN__OBJECT,
-            G_TYPE_BOOLEAN,
-            1,
-            G_TYPE_OBJECT);
+
 /**
  * EphyWebView::ge-favicon:
  * @embed:
@@ -843,14 +853,14 @@ ephy_web_view_class_init (EphyWebViewClass *klass)
             G_TYPE_POINTER);
 
 /**
- * EphyWebView::ge-search-key-press:
+ * EphyWebView::search-key-press:
  * @embed:
  * @event: the #GdkEventKey which triggered this signal
  *
- * The ::ge-search-key-press signal is emitted for keypresses which
+ * The ::search-key-press signal is emitted for keypresses which
  * should be used for find implementations.
  **/
-    g_signal_new ("ge-search-key-press",
+    g_signal_new ("search-key-press",
             EPHY_TYPE_WEB_VIEW,
             G_SIGNAL_RUN_LAST,
             G_STRUCT_OFFSET (EphyWebViewClass, search_key_press),
@@ -963,7 +973,7 @@ ge_new_window_cb (EphyWebView *view,
   g_return_if_fail (new_view != NULL);
 
   container = EPHY_EMBED_CONTAINER (gtk_widget_get_toplevel (GTK_WIDGET (new_view)));
-  g_return_if_fail (container != NULL || !GTK_WIDGET_TOPLEVEL (container));
+  g_return_if_fail (container != NULL || !gtk_widget_is_toplevel (GTK_WIDGET (container)));
 
   popups_manager_add_window (view, container);
 }
@@ -986,7 +996,7 @@ ephy_web_view_init (EphyWebView *web_view)
 
   priv = web_view->priv = EPHY_WEB_VIEW_GET_PRIVATE (web_view);
 
-  priv->address_expire = EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW;
+  priv->expire_address_now = TRUE;
   priv->is_blank = TRUE;
   priv->load_status = WEBKIT_LOAD_PROVISIONAL;
   priv->title = g_strdup (EMPTY_PAGE);
@@ -994,6 +1004,17 @@ ephy_web_view_init (EphyWebView *web_view)
   priv->security_level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
   priv->monitor_directory = FALSE;
 
+  priv->non_search_regex = g_regex_new ("(^localhost(\\.[^[:space:]]+)?(:\\d+)?(/.*)?$|"
+                                        "^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]$|"
+                                        "^::[0-9a-f:]*$|" /* IPv6 literals */
+                                        "^[0-9a-f:]+:[0-9a-f:]*$|" /* IPv6 literals */
+                                        "^[^\\.[:space:]]+\\.[^\\.[:space:]]+.*$|" /* foo.bar... */
+                                        "^https?://[^/\\.[:space:]]+.*$|"
+                                        "^about:.*$|"
+                                        "^data:.*$|"
+                                        "^file:.*$)",
+                                        G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
+
   g_signal_connect_object (web_view, "ge_document_type",
                            G_CALLBACK (ge_document_type_cb),
                            web_view, (GConnectFlags)0);
@@ -1069,13 +1090,42 @@ void
 ephy_web_view_load_url (EphyWebView *view,
                         const char *url)
 {
+  EphyWebViewPrivate *priv;
+  SoupURI *soup_uri = NULL;
   char *effective_url;
 
   g_return_if_fail (EPHY_IS_WEB_VIEW (view));
   g_return_if_fail (url);
 
-  effective_url = ephy_embed_utils_normalize_address (url);
-  webkit_web_view_open (WEBKIT_WEB_VIEW (view), effective_url);
+  priv = view->priv;
+
+  /* We use SoupURI as an indication of whether the value given in url
+   * is not something we want to search; we only do that, though, if
+   * the address has a web scheme, because SoupURI will consider any
+   * string: as a valid scheme, and we will end up prepending http://
+   * to it */
+  if (ephy_embed_utils_address_has_web_scheme (url))
+    soup_uri = soup_uri_new (url);
+
+  /* If the string doesn't look like an URI, let's search it; */
+  if (soup_uri == NULL &&
+      priv->non_search_regex &&
+      !g_regex_match (priv->non_search_regex, url, 0, NULL)) {
+    char *query_param = soup_form_encode ("q", url, NULL);
+    /* + 2 here is getting rid of 'q=' */
+    effective_url = g_strdup_printf (_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8"), query_param + 2);
+    g_free (query_param);
+  } else
+    effective_url = ephy_embed_utils_normalize_address (url);
+
+  if (g_str_has_prefix (effective_url, "javascript:"))
+    webkit_web_view_execute_script (WEBKIT_WEB_VIEW (view), effective_url);
+  else
+    webkit_web_view_open (WEBKIT_WEB_VIEW (view), effective_url);
+
+  if (soup_uri)
+    soup_uri_free (soup_uri);
+
   g_free (effective_url);
 }
 
@@ -1138,8 +1188,8 @@ ephy_web_view_set_address (EphyWebView *view,
   priv->is_blank = address == NULL ||
                    strcmp (address, "about:blank") == 0;
 
-  if (ephy_web_view_get_load_status (view) &&
-      priv->address_expire == EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW &&
+  if (ephy_web_view_is_loading (view) &&
+      priv->expire_address_now == TRUE &&
       priv->typed_address != NULL) {
     g_free (priv->typed_address);
     priv->typed_address = NULL;
@@ -1234,7 +1284,7 @@ ensure_page_info (EphyWebView *view, const char *address)
   EphyWebViewPrivate *priv = view->priv;
 
   if ((priv->address == NULL || priv->address[0] == '\0') &&
-      priv->address_expire == EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW) {
+      priv->expire_address_now == TRUE) {
     ephy_web_view_set_address (view, address);
   }
 
@@ -1332,11 +1382,10 @@ ephy_web_view_update_from_net_state (EphyWebView *view,
 
       g_object_freeze_notify (object);
 
-      priv->total_requests = 0;
-      priv->cur_requests = 0;
-
       ensure_page_info (view, uri);
 
+      priv->expire_address_now = TRUE;
+
       g_object_notify (object, "embed-title");
 
       g_object_thaw_notify (object);
@@ -1348,7 +1397,7 @@ ephy_web_view_update_from_net_state (EphyWebView *view,
       g_free (priv->loading_title);
       priv->loading_title = NULL;
 
-      priv->address_expire = EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW;
+      priv->expire_address_now = TRUE;
 
       g_object_notify (object, "embed-title");
 
@@ -1396,7 +1445,7 @@ ephy_web_view_file_monitor_reload_cb (EphyWebView *view)
     return TRUE;
   }
 
-  if (ephy_web_view_get_load_status (view)) {
+  if (ephy_web_view_is_loading (view)) {
     /* Wait a bit to reload if we're still loading! */
     priv->reload_delay_ticks = RELOAD_DELAY_MAX_TICKS / 2;
 
@@ -1603,48 +1652,64 @@ ephy_web_view_set_icon_address (EphyWebView *view,
  * @view: an #EphyWebView
  *
  * Returns whether @view can travel to a higher-level directory on the server.
- * For example, for http://www.example.com/subdir/index.html, returns %TRUE; for
- * http://www.example.com/index.html, returns %FALSE.
+ * For example, for http://www.example.com/subdir/foo.html, returns %TRUE; for
+ * http://www.example.com/, returns %FALSE.
  *
  * Return value: %TRUE if @view can browse to a higher-level directory
  **/
 gboolean
 ephy_web_view_can_go_up (EphyWebView *view)
 {
-  return FALSE;
+  SoupURI *uri;
+  gboolean result;
+
+  uri = soup_uri_new (ephy_web_view_get_address (view));
+  if (uri == NULL)
+    return FALSE;
+
+  if (strcmp (uri->scheme, "about") == 0 || strcmp (uri->scheme, "data") == 0) {
+    soup_uri_free (uri);
+    return FALSE;
+  }
+
+  result = (uri->fragment || uri->query || strlen (uri->path) > 1);
+  soup_uri_free (uri);
+
+  return result;
 }
 
 /**
- * ephy_web_view_get_load_status:
+ * ephy_web_view_is_loading:
  * @view: an #EphyWebView
  *
- * Returns whether the web page in @view has finished loading. A web page is
- * only finished loading after all images, styles, and other dependencies have
- * been downloaded and rendered.
+ * Returns whether the web page in @view has finished loading. A web
+ * page is only finished loading after all images, styles, and other
+ * dependencies have been downloaded and rendered, or when the load
+ * has failed for some reason.
  *
  * Return value: %TRUE if the page is still loading, %FALSE if complete
  **/
 gboolean
-ephy_web_view_get_load_status (EphyWebView *view)
+ephy_web_view_is_loading (EphyWebView *view)
 {
   WebKitLoadStatus status;
 
   status = webkit_web_view_get_load_status (WEBKIT_WEB_VIEW (view));
 
-  /* Workaround for webkit bug: https://bugs.webkit.org/show_bug.cgi?id=26409
-   * FIRST_VISUALLY_NON_EMPTY_LAYOUT might be emitted
-   * after LOAD_FINISHED. We just ignore any status
-   * other than WEBKIT_LOAD_PROVISIONAL once LOAD_FINISHED
-   * has been set, as WEBKIT_LOAD_PROVISIONAL probably means
-   * that webview has been reloaded.
+  /* FIRST_VISUALLY_NON_EMPTY_LAYOUT might be emitted after
+   * LOAD_FINISHED or LOAD_FAILED. We just ignore any status other
+   * than WEBKIT_LOAD_PROVISIONAL once LOAD_FINISHED or LOAD_FAILED
+   * have been set, as WEBKIT_LOAD_PROVISIONAL probably means that
+   * webview has started a new load.
    */
-  if (view->priv->load_status == WEBKIT_LOAD_FINISHED &&
+  if ((view->priv->load_status == WEBKIT_LOAD_FINISHED ||
+       view->priv->load_status == WEBKIT_LOAD_FAILED) &&
       status != WEBKIT_LOAD_PROVISIONAL)
     return FALSE;
 
   view->priv->load_status = status;
 
-  return status != WEBKIT_LOAD_FINISHED;
+  return status != WEBKIT_LOAD_FINISHED && status != WEBKIT_LOAD_FAILED;
 }
 
 const char *
@@ -1853,27 +1918,23 @@ ephy_web_view_get_typed_address (EphyWebView *view)
  * ephy_web_view_set_typed_address:
  * @view: an #EphyWebView
  * @address: the new typed address, or %NULL to clear it
- * @expire: when to expire this address_expire
  * 
  * Sets the text that @view's #EphyWindow will display in its location toolbar
  * entry when @view is selected.
  **/
 void
 ephy_web_view_set_typed_address (EphyWebView *view,
-                                 const char *address,
-                                 EphyWebViewAddressExpire expire)
+                                 const char *address)
 {
   EphyWebViewPrivate *priv = EPHY_WEB_VIEW (view)->priv;
 
   g_free (priv->typed_address);
   priv->typed_address = g_strdup (address);
-
-  if (expire == EPHY_WEB_VIEW_ADDRESS_EXPIRE_CURRENT &&
-      !ephy_web_view_get_load_status (view)) {
-    priv->address_expire = EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW;
-  } else {
-    priv->address_expire = expire;
-  }
+  /* If the page is loading prevent the typed address from going away,
+     since Epiphany will try to overwrite the typed address with the
+     confirmed full URL when passing through, for example, the
+     COMMITTED state. */
+  priv->expire_address_now = !ephy_web_view_is_loading (view);
 
   g_object_notify (G_OBJECT (view), "typed-address");
 }
@@ -1958,16 +2019,10 @@ ephy_web_view_get_security_level (EphyWebView *view,
                                   EphyWebViewSecurityLevel *level,
                                   char **description)
 {
-  if (level) {
-    const gchar *uri = ephy_web_view_get_address (view);
+  g_return_if_fail (EPHY_IS_WEB_VIEW (view));
 
-    /* FIXME: as a temporary workaround, determine security level
-       based on the existence of a 'https' prefix for the URI */
-    if (uri && g_str_has_prefix(uri, "https"))
-      *level = EPHY_WEB_VIEW_STATE_IS_SECURE_HIGH;
-    else
-      *level = EPHY_WEB_VIEW_STATE_IS_UNKNOWN;
-  }
+  if (level)
+    *level = view->priv->security_level;
 }
 
 /**
@@ -2041,7 +2096,54 @@ ephy_web_view_print_preview_navigate (EphyWebView *view,
 GSList *
 ephy_web_view_get_go_up_list (EphyWebView *view)
 {
-  return NULL;
+  SoupURI *uri;
+  GSList *result = NULL;
+  char *t1, *t2;
+
+  uri = soup_uri_new (ephy_web_view_get_address (view));
+  if (uri == NULL)
+    return NULL;
+
+  if (strcmp (uri->scheme, "about") == 0 || strcmp (uri->scheme, "data") == 0) {
+    soup_uri_free (uri);
+    return NULL;
+  }
+
+  /* remove fragment, then query, then go up path */
+  if (uri->fragment) {
+    soup_uri_set_fragment (uri, NULL);
+    result = g_slist_prepend (result, soup_uri_to_string (uri, FALSE));
+  }
+
+  if (uri->query) {
+    soup_uri_set_query (uri, NULL);
+    result = g_slist_prepend (result, soup_uri_to_string (uri, FALSE));
+  }
+
+  if (uri->path[strlen(uri->path)-1] != '/') {
+    /* not a trailing slash, remove "file" part */
+    t1 = strrchr (uri->path, '/');
+    t2 = g_strndup (uri->path, t1-uri->path+1);
+    soup_uri_set_path (uri, t2);
+    g_free (t2);
+    result = g_slist_prepend (result, soup_uri_to_string (uri, FALSE));
+  }
+
+  while (strcmp(uri->path, "/") != 0) {
+    /* chop trailing / */
+    uri->path[strlen (uri->path)-1] = 0;
+    t1 = strrchr (uri->path, '/');
+    t2 = g_strndup (uri->path, t1-uri->path+1);
+    soup_uri_set_path (uri, t2);
+    g_free (t2);
+    result = g_slist_prepend (result, soup_uri_to_string (uri, FALSE));
+  }
+
+  result = g_slist_reverse (result);
+
+  soup_uri_free (uri);
+
+  return result;
 }
 
 /**
@@ -2064,7 +2166,7 @@ ephy_web_view_get_title_composite (EphyWebView *view)
 
   g_return_val_if_fail (EPHY_IS_WEB_VIEW (view), NULL);
 
-  is_loading = ephy_web_view_get_load_status (view);
+  is_loading = ephy_web_view_is_loading (view);
   is_blank = ephy_web_view_get_is_blank (view);
   loading_title = ephy_web_view_get_loading_title (view);
   title = ephy_web_view_get_title (view);
@@ -2080,3 +2182,247 @@ ephy_web_view_get_title_composite (EphyWebView *view)
   return title != NULL ? title : "";
 }
 
+static void
+ephy_web_view_save_sub_resource_start (GList *subresources, char *destination_uri);
+
+static void
+ephy_web_view_close_cb (GOutputStream *ostream, GAsyncResult *result, GString *data)
+{
+  GList *subresources;
+  char *sub_destination_uri;
+  GError *error = NULL;
+
+  subresources = (GList*)g_object_get_data (G_OBJECT (ostream),
+                                            "ephy-web-view-save-subresources");
+
+  sub_destination_uri = (char*)g_object_get_data (G_OBJECT (ostream),
+                                                  "ephy-web-view-save-dest-uri");
+
+  g_output_stream_close_finish (ostream, result, &error);
+  g_object_unref (ostream);
+
+  if (error) {
+    g_list_free (subresources);
+    g_free (sub_destination_uri);
+    g_warning ("Unable to write to file: %s", error->message);
+    g_error_free (error);
+    return;
+  }
+
+  if (!subresources || !subresources->next) {
+    g_list_free (subresources);
+    g_free (sub_destination_uri);
+    return;
+  }
+
+  subresources = subresources->next;
+  ephy_web_view_save_sub_resource_start (subresources, sub_destination_uri);
+}
+
+static void
+ephy_web_view_save_write_cb (GOutputStream *ostream, GAsyncResult *result, GString *data)
+{
+  GError *error = NULL;
+  gssize written;
+
+  written = g_output_stream_write_finish (ostream, result, &error);
+  if (error) {
+    GList *subresources;
+    char *sub_destination_uri;
+
+    subresources = (GList*)g_object_get_data (G_OBJECT (ostream),
+                                             "ephy-web-view-save-subresources");
+    g_list_free (subresources);
+
+    sub_destination_uri = (char*)g_object_get_data (G_OBJECT (ostream),
+                                                    "ephy-web-view-save-dest-uri");
+    g_free (sub_destination_uri);
+
+    g_string_free (data, FALSE);
+    g_object_unref (ostream);
+
+    g_warning ("Unable to write to file: %s", error->message);
+
+    g_error_free (error);
+    return;
+  }
+
+  if (written == data->len) {
+    g_string_free (data, FALSE);
+    g_output_stream_close_async (ostream, G_PRIORITY_DEFAULT, NULL,
+                                 (GAsyncReadyCallback)ephy_web_view_close_cb,
+                                 NULL);
+    return;
+  }
+
+  data->len -= written;
+  data->str += written;
+
+  g_output_stream_write_async (ostream,
+                               data->str, data->len,
+                               G_PRIORITY_DEFAULT, NULL,
+                               (GAsyncReadyCallback)ephy_web_view_save_write_cb,
+                               data);
+}
+
+static void
+ephy_web_view_save_replace_cb (GFile *file, GAsyncResult *result, GString *const_data)
+{
+  GFileOutputStream *ostream;
+  GList *subresources;
+  char *sub_destination_uri;
+  GString *data;
+  GError *error = NULL;
+
+  subresources = (GList*)g_object_get_data (G_OBJECT (file),
+                                            "ephy-web-view-save-subresources");
+
+  sub_destination_uri = (char*)g_object_get_data (G_OBJECT (file),
+                                                  "ephy-web-view-save-dest-uri");
+
+  ostream = g_file_replace_finish (file, result, &error);
+  if (error) {
+    g_warning ("Failed to save page: %s", error->message);
+    g_list_free (subresources);
+    g_free (sub_destination_uri);
+    g_error_free (error);
+    return;
+  }
+
+  if (const_data) {
+    data = g_string_sized_new (const_data->len);
+    data->str = const_data->str;
+    data->len = const_data->len;
+  } else
+    data = g_string_new ("");
+
+  /* If we have subresources to handle, pass the information along */
+  if (subresources) {
+    g_object_set_data (G_OBJECT (ostream),
+                       "ephy-web-view-save-subresources",
+                       subresources);
+
+    g_object_set_data (G_OBJECT (ostream),
+                       "ephy-web-view-save-dest-uri",
+                       sub_destination_uri);
+  }
+
+  g_output_stream_write_async (G_OUTPUT_STREAM (ostream),
+                               data->str, data->len,
+                               G_PRIORITY_DEFAULT, NULL,
+                               (GAsyncReadyCallback)ephy_web_view_save_write_cb,
+                               data);
+}
+
+static void
+ephy_web_view_save_sub_resource_start (GList *subresources, char *destination_uri)
+{
+  WebKitWebResource *resource;
+  GFile *file;
+  char *resource_uri;
+  char *resource_name;
+  const GString *data;
+
+  resource = WEBKIT_WEB_RESOURCE (subresources->data);
+
+  resource_uri = (char*)webkit_web_resource_get_uri (resource);
+  resource_name = g_path_get_basename (resource_uri);
+
+  resource_uri = g_strdup_printf ("%s/%s",
+                                  destination_uri,
+                                  resource_name);
+
+  file = g_file_new_for_uri (resource_uri);
+  g_free (resource_uri);
+
+  g_object_set_data (G_OBJECT (file),
+                     "ephy-web-view-save-dest-uri",
+                     destination_uri);
+
+  g_object_set_data (G_OBJECT (file),
+                     "ephy-web-view-save-subresources",
+                     subresources);
+
+  data = webkit_web_resource_get_data (resource);
+
+  g_file_replace_async (file, NULL, FALSE,
+                        G_FILE_CREATE_REPLACE_DESTINATION|G_FILE_CREATE_PRIVATE,
+                        G_PRIORITY_DEFAULT, NULL,
+                        (GAsyncReadyCallback)ephy_web_view_save_replace_cb,
+                        (GString*)data);
+
+  g_object_unref (file);
+}
+
+static void
+ephy_web_view_save_sub_resources (EphyWebView *view, const char *uri, GList *subresources)
+{
+  GFile *file;
+  char *filename;
+  char *dotpos;
+  char *directory_uri;
+  char *tmp;
+  char *destination_uri;
+  GError *error = NULL;
+
+  /* filename of the main resource without extension */
+  filename = g_path_get_basename (uri);
+  dotpos = g_strrstr (filename, ".");
+  if (dotpos)
+    *dotpos = '\0';
+
+  directory_uri = g_path_get_dirname (uri);
+
+  /* Translators: this is the directory name to store auxilary files
+   * when saving html files.
+   */
+  tmp = g_strdup_printf (_("%s Files"), filename);
+  destination_uri = g_strdup_printf ("%s/%s", directory_uri, tmp);
+  g_free (tmp);
+
+  g_free (filename);
+  g_free (directory_uri);
+
+  file = g_file_new_for_uri (destination_uri);
+  if (!g_file_make_directory (file, NULL, &error)) {
+    g_warning ("Could not create directory: %s", error->message);
+    g_error_free (error);
+    g_object_unref (file);
+    return;
+  }
+  g_object_unref (file);
+
+  /* Now, let's start saving sub resources */
+  ephy_web_view_save_sub_resource_start (subresources, destination_uri);
+}
+
+void
+ephy_web_view_save (EphyWebView *view, const char *uri)
+{
+  WebKitWebFrame *frame;
+  WebKitWebDataSource *data_source;
+  GList *subresources;
+  const GString *data;
+  GFile *file;
+
+  /* Save main resource */
+  frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW(view));
+  data_source = webkit_web_frame_get_data_source (frame);
+  data = webkit_web_data_source_get_data (data_source);
+
+  file = g_file_new_for_uri (uri);
+  g_file_replace_async (file, NULL, FALSE,
+                        G_FILE_CREATE_REPLACE_DESTINATION|G_FILE_CREATE_PRIVATE,
+                        G_PRIORITY_DEFAULT, NULL,
+                        (GAsyncReadyCallback)ephy_web_view_save_replace_cb,
+                        (GString*)data);
+
+  g_object_unref (file);
+
+  /* If subresources exist, save them */
+  subresources = webkit_web_data_source_get_subresources (data_source);
+  if (!subresources)
+    return;
+
+  ephy_web_view_save_sub_resources (view, uri, subresources);
+}
