@@ -247,6 +247,12 @@ ephy_dot_dir (void)
 	return dot_dir;
 }
 
+gboolean
+ephy_has_private_profile (void)
+{
+	return have_private_profile;
+}
+
 gboolean
 ephy_file_helpers_init (const char *profile_dir,
 			gboolean private_profile,
