@@ -14,11 +14,14 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-langs.h 6588 2006-09-13 11:34:25Z chpe $
  */
 
+#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
+#error "Only <epiphany/epiphany.h> can be included directly."
+#endif
+
 #ifndef EPHY_LANGS_H
 #define EPHY_LANGS_H
 
@@ -29,16 +32,6 @@ G_BEGIN_DECLS
 #define ISO_639_DOMAIN	"iso_639"
 #define ISO_3166_DOMAIN	"iso_3166"
 
-typedef struct
-{
-	char *title;
-	char *code;
-} EphyFontsLanguageInfo;
-
-const EphyFontsLanguageInfo *ephy_font_languages	 (void);
-
-guint			     ephy_font_n_languages	 (void);
-
 void			     ephy_langs_append_languages (GArray *array);
 
 void			     ephy_langs_sanitise	 (GArray *array);
