@@ -139,7 +139,7 @@ embed_net_stop_cb (EphyWebView *view,
 		   GParamSpec *pspec,
 		   EphyEncodingDialog *dialog)
 {
-	if (ephy_web_view_get_load_status (view) == FALSE)
+	if (ephy_web_view_is_loading (view) == FALSE)
 		sync_encoding_against_embed (dialog);
 }
 
@@ -291,7 +291,7 @@ automatic_toggled_cb (GtkToggleButton *button, EphyEncodingDialog *dialog)
 static void
 ephy_encoding_dialog_init (EphyEncodingDialog *dialog)
 {
-	GtkWidget *treeview, *scroller, *button, *window;
+	GtkWidget *treeview, *scroller, *button, *window, *child;
 	GtkTreeSelection *selection;
 	EphyNode *node;
 
@@ -345,13 +345,15 @@ ephy_encoding_dialog_init (EphyEncodingDialog *dialog)
 			(EPHY_DIALOG (dialog), properties[SCROLLED_WINDOW_PROP].id);
 	gtk_container_add (GTK_CONTAINER (scroller), treeview);
 
+	child = gtk_bin_get_child (GTK_BIN (button));
+
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_BIN (button)->child), TRUE);
+	gtk_label_set_use_markup (GTK_LABEL (child), TRUE);
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (automatic_toggled_cb), dialog);
 
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[MANUAL_PROP].id);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_BIN (button)->child), TRUE);
+	gtk_label_set_use_markup (GTK_LABEL (child), TRUE);
 
 	dialog->priv->enc_view = treeview;
 
