@@ -16,9 +16,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-shell.h 7580 2007-10-28 13:02:28Z chpe $
  */
 
+#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
+#error "Only <epiphany/epiphany.h> can be included directly."
+#endif
+
 #ifndef EPHY_SHELL_H
 #define EPHY_SHELL_H
 
@@ -27,6 +30,7 @@
 #include "ephy-window.h"
 #include "ephy-embed.h"
 
+#include <webkit/webkit.h>
 #include <glib-object.h>
 #include <glib.h>
 
@@ -65,6 +69,7 @@ typedef enum
 
 	/* The way to load */
 	EPHY_NEW_TAB_ALLOW_FIXUP	= 1 << 12,
+	EPHY_NEW_TAB_FROM_EXTERNAL      = 1 << 13,
 	
 } EphyNewTabFlags;
 
@@ -96,9 +101,9 @@ EphyEmbed      *ephy_shell_new_tab			(EphyShell *shell,
 EphyEmbed      *ephy_shell_new_tab_full			(EphyShell *shell,
 							 EphyWindow *parent_window,
 							 EphyEmbed *previous_embed,
-							 const char *url,
+							 WebKitNetworkRequest *request,
 							 EphyNewTabFlags flags,
-							 EphyEmbedChrome chrome,
+							 EphyWebViewChrome chrome,
 							 gboolean is_popup,
 							 guint32 user_time);
 
