@@ -149,7 +149,7 @@ maybe_finish_activation_cb (EphyWindow *window,
 
 	while (widget != NULL && widget != wtoolbar)
 	{
-		widget = widget->parent;
+		widget = gtk_widget_get_parent (widget);
 	}
 
 	/* if widget == toolbar, the new focus widget is in the toolbar, so we
@@ -183,8 +183,7 @@ sync_user_input_cb (EphyLocationAction *action,
 	address = ephy_location_action_get_address (action);
 
 	priv->updating_address = TRUE;
-	ephy_web_view_set_typed_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), address,
-                                         EPHY_WEB_VIEW_ADDRESS_EXPIRE_CURRENT);
+	ephy_web_view_set_typed_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), address);
 	priv->updating_address = FALSE;
 }
 
