@@ -15,10 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
-#include <gtk/gtkaction.h>
+#include <gtk/gtk.h>
 
 #include "ephy-window.h"
 
