@@ -13,12 +13,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: window-commands.h,v 1.27 2006/09/24 19:59:37 jframeau Exp $
  */
 
-#include <gtk/gtkaction.h>
+#include <gtk/gtk.h>
 
 #include "ephy-window.h"
 
